from .linear_regression import LinearRegression, PolynomialRegression, BasisFunctionRegression
from .linear_regression_errors import LinearRegressionwithErrors

from .kernel_regression import NadarayaWatson
from .TLS import TLS_logL
